local LUI, T, E, L, V, P, G = unpack(select(2, ...))
if LUI:CheckDB("db", "modules", "tooltip", "noShiftKeyDown") then return end
local LNSK = E:NewModule("LuiNoShiftKeyDown")
local TT = E:GetModule('Tooltip')

function LNSK:Initialize()
    local targetList = {}
    local C_MountJournal_GetMountInfoExtraByID = C_MountJournal.GetMountInfoExtraByID
    local C_PetBattles_IsInBattle = C_PetBattles.IsInBattle

    function TT:GameTooltip_OnTooltipSetUnit(tt)
        if tt:IsForbidden() then return end
    
        local unit = select(2, tt:GetUnit())
        local isShiftKeyDown = IsShiftKeyDown()
        local isControlKeyDown = IsControlKeyDown()
        local isPlayerUnit = UnitIsPlayer(unit)
        if((tt:GetOwner() ~= _G.UIParent) and (self.db.visibility and self.db.visibility.unitFrames ~= 'NONE')) then
            local modifier = self.db.visibility.unitFrames
    
            if(modifier == 'ALL' or not ((modifier == 'SHIFT' and isShiftKeyDown) or (modifier == 'CTRL' and isControlKeyDown) or (modifier == 'ALT' and IsAltKeyDown()))) then
                tt:Hide()
                return
            end
        end
    
        if not unit then
            local GMF = GetMouseFocus()
            if GMF and GMF.GetAttribute and GMF:GetAttribute("unit") then
                unit = GMF:GetAttribute("unit")
            end
            if not unit or not UnitExists(unit) then
                return
            end
        end
    
        self:RemoveTrashLines(tt)
    
        local color = self:SetUnitText(tt, unit, UnitLevel(unit), isShiftKeyDown)
    
        if self.db.showMount and not isShiftKeyDown and unit ~= "player" and isPlayerUnit then
            for i = 1, 40 do
                local name, _, _, _, _, _, _, _, _, id = UnitBuff(unit, i)
                if not name then break end
    
                if self.MountIDs[id] then
                    local _, _, sourceText = C_MountJournal_GetMountInfoExtraByID(self.MountIDs[id])
                    tt:AddDoubleLine(format("%s:", _G.MOUNT), name, nil, nil, nil, 1, 1, 1)
    
                    if sourceText and isControlKeyDown then
                        local sourceModified = sourceText:gsub("|n", "\10")
                        for x in gmatch(sourceModified, '[^\10]+\10?') do
                            local left, right = strmatch(x, '(.-|r)%s?([^\10]+)\10?')
                            if left and right then
                                tt:AddDoubleLine(left, right, nil, nil, nil, 1, 1, 1)
                            else
                                tt:AddDoubleLine(_G.FROM, sourceText:gsub('|c%x%x%x%x%x%x%x%x',''), nil, nil, nil, 1, 1, 1)
                            end
                        end
                    end
    
                    break
                end
            end
        end
    
        if not isShiftKeyDown and not isControlKeyDown then
            local unitTarget = unit.."target"
            if self.db.targetInfo and unit ~= "player" and UnitExists(unitTarget) then
                local targetColor
                if(UnitIsPlayer(unitTarget) and not UnitHasVehicleUI(unitTarget)) then
                    local _, class = UnitClass(unitTarget)
                    targetColor = _G.CUSTOM_CLASS_COLORS and _G.CUSTOM_CLASS_COLORS[class] or _G.RAID_CLASS_COLORS[class]
                else
                    targetColor = E.db.tooltip.useCustomFactionColors and E.db.tooltip.factionColors[UnitReaction(unitTarget, "player")] or _G.FACTION_BAR_COLORS[UnitReaction(unitTarget, "player")]
                end
    
                tt:AddDoubleLine(format("%s:", _G.TARGET), format("|cff%02x%02x%02x%s|r", targetColor.r * 255, targetColor.g * 255, targetColor.b * 255, UnitName(unitTarget)))
            end
    
            if self.db.targetInfo and IsInGroup() then
                for i = 1, GetNumGroupMembers() do
                    local groupUnit = (IsInRaid() and "raid"..i or "party"..i);
                    if (UnitIsUnit(groupUnit.."target", unit)) and (not UnitIsUnit(groupUnit,"player")) then
                        local _, class = UnitClass(groupUnit);
                        local classColor = _G.CUSTOM_CLASS_COLORS and _G.CUSTOM_CLASS_COLORS[class] or _G.RAID_CLASS_COLORS[class]
                        if not classColor then classColor = _G.RAID_CLASS_COLORS.PRIEST end
                        tinsert(targetList, format("|c%s%s|r", classColor.colorStr, UnitName(groupUnit)))
                    end
                end
                local numList = #targetList
                if (numList > 0) then
                    tt:AddLine(format("%s (|cffffffff%d|r): %s", L["Targeted By:"], numList, tconcat(targetList, ", ")), nil, nil, nil, true);
                    wipe(targetList);
                end
            end
        end
    
        if isPlayerUnit then
            self:AddInspectInfo(tt, unit, 0, color.r, color.g, color.b)
        end
    
        if unit and self.db.npcID and not isPlayerUnit then
            if C_PetBattles_IsInBattle() then return end
            local guid = UnitGUID(unit) or ""
            local id = tonumber(guid:match("%-(%d-)%-%x-$"), 10)
            if id then
                tt:AddLine(("|cFFCA3C3C%s|r %d"):format(_G.ID, id))
            end
        end
    
        if color then
            tt.StatusBar:SetStatusBarColor(color.r, color.g, color.b)
        else
            tt.StatusBar:SetStatusBarColor(0.6, 0.6, 0.6)
        end
    
        local textWidth = tt.StatusBar.text:GetStringWidth()
        if textWidth then
            tt:SetMinimumWidth(textWidth)
        end
    end
end

local function InitializeCallback()
    if not E.db.lui.modules.tooltip["noShiftKeyDown"] then return end
    LNSK:Initialize()
end

E:RegisterModule(LNSK:GetName(), InitializeCallback)